# CVE connector

CVE connector is CRUSOE component which obtains data about CVEs from the National Vulnerability Database (NVD)
and adds additional information obtained from software vendors.

## Design
The component consists of two subcomponent: `nvd_cve` and `vendor_cve`.

##### Subcomponent `nvd_cve`
This part of CVE connector is responsible for download of CVE data feeds from the NVD, their parsing, assigning 
predicted impact and storing to the database.

1. Folder `categorization` contains implementation of categorization algorithm split among several modules.
2. Module `CVE_parser.py` contains parser of CVE data provided by the NVD.
3. Module `toneo4j.py` contains functionality which adds parsed data to Neo4j database according to CPEs present 
in the database.
4. Module `utility.py` contains functionality which downloads and deletes CVE data feed from the NVD.

##### Subcomponent `vendor_cve`
This part of CVE connector is responsible for download of data about CVEs from vendor websites (supported vendors are
Adobe, Android, Apple, Cisco, Lenovo, Microsoft, Oracle, RedHat). 

1. Folder `data` contains configuration file with URLs for vendor websites.
2. Folder `parsers` contains:
    * general parser and parsers for specific types of files (HTML, JSON and XML)
    * parsers for specific vendors - each folder contains parsers for data provided by vendors. Usually there is parser 
    for main website (ending with `MainPageParser.py`) which links to specific websites for individual vulnerabilities 
    (ending with `VulnerabilityParser.py`). 
3. Folder `storing_to_database` contains module `neo4j_storing.py` responsible for addition of CVEs to the database.
4. Folder `utilities` contains several helper functions.
5. Folder `vendors_storage_structures` contains for each vendor class that stores its information.
6. Folder `vulnerability_metrics` contains functionality related to CVSSv2 and CVSSv3, respectively.
7. Module `main.py` contains entry-point functionality for processing data from vendors.

## Required packages/versions

At least `Python3.6`. 

Required packages are be specified in `setup.py` and they will be installed when one of following installation methods is used. 

Further, `neo4j-client` (another CRUSOE component) is required, see https://gitlab.ics.muni.cz/CRUSOE/neo4j-client. 

## Usage

### Install

You can install CVE connector with pip from local directory via:

```bash
$ pip install . -r requirements.txt
```

### Running

    **Data from NVD**

```python
from cve_connector.nvd_cve import toneo4j
from datetime import datetime, timedelta
specified_time = (datetime.now() - timedelta(days=1)).isoformat()
toneo4j.move_cve_data_to_neo4j(specified_time, "neo4jpassword", "/tmp/cve-data")
```

    **Vendor data**
    
```python
from cve_connector.vendor_cve.implementation.main import add_vendor_cves
add_vendor_cves("/tmp/ms_downloads", "neo4jpassword")
```
