"""Module contains helper functions that are used in vulnerability classification."""


def cve_is_about_system(cpe_type):
    """
    Determines whether CVE is about system.

    :param cpe_type: One of {'a', 'o', 'h'} = application, operating system,
    hardware
    :return: true if CVE is about system.
    """
    return ('o' in cpe_type or 'h' in cpe_type) and 'a' not in cpe_type


def cve_is_about_application(cpe_type):
    """
    Determines whether CVE is about application.

    :param cpe_type: One of {'a', 'o', 'h'} = application, operating system,
    hardware
    :return: true if description is about application.
    """
    return 'a' in cpe_type


def test_incidence(description, list_of_keywords):
    """
    Determines whether at least one of keyword from list_of_keywords is in
    description.

    :param description: description of CVE.
    :param list_of_keywords: keywords function searches for
    :return: true if one of keywords is in description.
    """
    for word in list_of_keywords:
        if word in description:
            return True

    return False
