"""Module contains class for storing information about CVEs from vendor Android."""

from cve_connector.vendor_cve.implementation.vendors_storage_structures.general_vendor import Vendor

from cve_connector.vendor_cve.implementation.utilities.check_correctness import is_correct_cve_id


class Android(Vendor):
    """
    Class for storing information about CVEs from vendor Android.
    """
    def __init__(self, cve='', aff_comp='', refer='', vuln_type='', sev='', upd_ver='',
                 comp='', published=None):
        super().__init__()
        self.cve_id = cve
        self.affected_component = aff_comp
        self.description = refer
        self.vulnerability_type = vuln_type
        self.severity = sev
        self.updated_aosp_versions = upd_ver
        self.component = comp
        self.published = published

        self.patch_available = True

    def is_valid_entity(self):
        """
        Tests correctness of created instance of this class.

        :return: True if valid
        """
        return is_correct_cve_id(self.cve_id)
