"""Module contains class for storing information about CVEs from vendor Lenovo."""

from cve_connector.vendor_cve.implementation.vendors_storage_structures.general_vendor import Vendor

from cve_connector.vendor_cve.implementation.utilities.check_correctness import is_correct_cve_id


class Lenovo(Vendor):
    """
    Class for storing information about CVEs from vendor Lenovo.
    """
    def __init__(self, cve_id='', vuln_name='', pot_impact='', severity='', scope='', desc='',
                 mit_strat='', fst_pub='', patch=False):
        super().__init__()
        self.vulnerability_name = vuln_name
        self.potential_impact = pot_impact
        self.severity = severity
        self.scope_of_impact = scope
        self.cve_id = cve_id
        self.description = desc
        self.mitigation_strategy_for_cust = mit_strat

        self.published = fst_pub

        self.patch_available = patch

    def is_valid_entity(self):
        """
        Tests correctness of created instance of this class.

        :return: True if valid
        """
        return is_correct_cve_id(self.cve_id)
