"""Module contains class for storing information about CVEs from vendor RedHat."""

from cve_connector.vendor_cve.implementation.vendors_storage_structures.general_vendor import Vendor

from cve_connector.vendor_cve.implementation.utilities.check_correctness \
    import is_correct_cve_id, is_correct_score


class RedHat(Vendor):
    """
    Class for storing information about CVEs from vendor RedHat.
    """
    def __init__(self, cve='', desc='', imp='', pub_date='', statement='', bugz_desc=''):
        super().__init__()
        self.cve_id = cve
        self.description = desc
        self.impact = imp
        self.published = pub_date
        self.cwes = []
        self.statement = statement

        self.cvss_v3 = None
        self.cvss_v2 = None

        self.bugzilla_description = bugz_desc

        self.patch_available = False

    def is_valid_entity(self):
        """
        Tests correctness of created instance of this class.

        :return: True if valid
        """
        return is_correct_cve_id(self.cve_id) and (self.cvss_v3.base_score == '' or
                                                   is_correct_score(self.cvss_v3.base_score))
