#!/usr/bin/env python

"""
Module which cleans old,corrupted files.
"""

import os
import time
import structlog

def remove_csv_before(location, logger=structlog.get_logger()):
    """
    Remove all csv which are older than ~12 hours
    :param location: path to the directory
    :param logger: logger for the method
    :return: None
    """
    for files in os.listdir(location):
        if files.endswith("masaryk_university-ip.csv"):
            if time.time() - os.path.getmtime(location + files) > 43200:
                logger.info(f'Removing file: {location}{files}')
                os.remove(location + files)

def remove_csv_after(location, logger=structlog.get_logger()):
    """
    Remove all csv without time limit.
    :param location: path to the directory
    :param logger: logger for the method
    :return: None
    """
    for files in os.listdir(location):
        if files.endswith("masaryk_university-ip.csv"):
            logger.info(f'Removing file: {location}{files}')
            os.remove(location + files)

def remove_corrupted_files(location, logger=structlog.get_logger()):
    """
    Remove corrupted files, to be specific-> files with no content
    :param location: path to the directory with files
    :param logger: logger for the method
    :return: None
    """
    for file_scan in os.listdir(location):
        path = os.path.join(location, file_scan)
        filename, file_extension = os.path.splitext(path)
        if file_extension == ".csv" and "masaryk_university-ip" in filename:
            modified = time.time() - os.stat(path).st_mtime
            if os.stat(path).st_size == 0 and modified < 400:
                logger.info(f'Removing empty file {file_scan}')
                os.remove(path)
